// ----UltraSone-----
int Metingen = 10;
int PingDelay = 25;
const int trigPin = 7;
const int echoPin = 6;
int MaxDistance = 50;
// -----End UltraSone-----

// -----TFT-----
#include <TFT.h>  
#include <SPI.h>
//Pin definition
#define cs   10
#define dc   9
#define rst  8
//Create instance
TFT Screen = TFT(cs, dc, rst);
//Setup some useful variables
double Deg2Rad = 0.0174532925;
double Rad2Deg = 57.2957795;
// -----End TFT-----

// -----Stepper-----
// defines pins numbers
int Rotation=200;
int stepPin = 3;
int dirPin = 4;
bool CW = false;
// -----End Stepper

void setup() {
  //Sonar setup
  pinMode(trigPin, OUTPUT);
  pinMode(echoPin, INPUT);

  //Setup TFT
  Screen.begin();
  Screen.stroke(0, 255, 0);

  //Stepper setup
  pinMode(stepPin,OUTPUT); 
  pinMode(dirPin,OUTPUT);  

  Serial.begin(9600);
}

void loop() {
  //Clear Screen
  Screen.background(0, 0, 0);
  Screen.stroke(0, 0, 255);
  Screen.circle(80, 64, 63);
  Screen.stroke(0, 255, 0);

  //Set direction to CCW
  digitalWrite(dirPin,HIGH);
  for(int x = 0; x < Rotation; x++) {
    digitalWrite(stepPin,HIGH); 
    delayMicroseconds(1000); 
    digitalWrite(stepPin,LOW); 
    delayMicroseconds(1000);
    
    Serial.print(x);
    
    //Get the distance measurements
    double Distance=SonarPing();
    double Degrees=x*(360.0/Rotation);
    //Draw a point for every step
    DrawPoint(Distance, Degrees);
  }

  //Set direction to CW
  digitalWrite(dirPin,LOW);
  for(int x = 0; x < Rotation; x++) {
    digitalWrite(stepPin,HIGH);
    delayMicroseconds(1000);
    digitalWrite(stepPin,LOW);
    delayMicroseconds(1000);
    delay(2);
  }
}

double SonarPing() 
{
  //Create Empty results array
  int Res[Metingen];
  for (int i=0; i<Metingen; i++) {
    long tijd, cm;
    //Start with a low to get a clean 10 us high pulse to trigger the sensor
    digitalWrite(trigPin, LOW);
    delayMicroseconds(2);
    digitalWrite(trigPin, HIGH);
    delayMicroseconds(10);
    digitalWrite(trigPin, LOW);
    // Wait for high
    tijd = pulseIn(echoPin, HIGH);
    //Convert time to distance
    cm = tijd / 29 / 2;
    //Ignore too high results
    Res[i]=cm;
    delay(PingDelay);
  }

  int Sum=0;
  int RealMeasurements=0;
  for (int a = 1; a<Metingen; a++) {
    if (Res[a]>0) {
      if (Res[a]>MaxDistance) {
        Res[a]=MaxDistance;
      }
      RealMeasurements++;
      Sum=Sum+Res[a];  
    }
  }
  double AVG=Sum/RealMeasurements;
  Serial.print(": "+String(AVG)+"  ,  ");
  return AVG;
}

void DrawPoint(double Distance, double Degrees) {
  //Convert degrees to radians
  double Rad = Degrees * Deg2Rad;
  //Calculate and round coordinates
  int X = Round((cos(Rad) * (Distance))/(MaxDistance/62.0)); 
  int Y = Round((sin(Rad) * (Distance))/(MaxDistance/62.0));
  Serial.println(Degrees);
  //Draw the point on screen
  Screen.point(X+80, 64-Y);
}

//Simple rounding function
int Round(double In) {
  //If remainder < 0.5 just convert it to an int
  if (fmod(In, 1)< 0.5) {
    return int(In);
  }
  //Else convert it to an int and add one to compensate
  else {
    return int(In)+1;
  }
}
